# {% xhr %} template tag.
# Skip rendering, but replace output with placeholder which will load the content
# through AJAX later on.

# {% xhr %}
#    ...
# {% else %}
#    ...
# {% endxhr %}
from django.template import Library, Node, loader
from django.conf import settings

register = Library()


class XhrNode(Node):
    def __init__(self, xhr_nodelist, else_nodelist):
        self.xhr_nodelist = xhr_nodelist
        self.else_nodelist = else_nodelist

    def render(self, context):
        container = '<div class="xhr_container">%s</div>'

        # When this is a XHR request, render the real content
        if 'request' in context and ('xhr' in context['request'].GET or 'xhr' in context['request'].POST):
            return container % self.xhr_nodelist.render(context)
        # Otherwise, render a loading placeholder
        else:
            if self.else_nodelist:
                return container % self.else_nodelist.render(context)
            else:
                template = getattr(settings, 'XHR_LOADING_TEMPLATE', 'django-ajax/_loader.html')
                return container % loader.get_template(template).render()


@register.tag
def xhr(parser, token):
    # Nodelist
    xhr_nodelist = parser.parse(('endxhr', 'else'))
    else_nodelist = None

    if parser.tokens[0].contents == 'else':
        parser.delete_first_token()

        else_nodelist = parser.parse(('endxhr',))
        parser.delete_first_token()
    else:
        parser.delete_first_token()

    # Return meta node
    return XhrNode(xhr_nodelist, else_nodelist)



# {% tabbing %} template tag.
# Tab page generation

# {% tabpage %}   | {% tabpage no_ajax %}
#   {% tabs %}
#         <ul>
#           <li><a href="..."></a></li>
#           <li><a href="..."></a></li>
#         </ul>
#
#        {% tab "name" %}<a href="..."> ...</a>{% endtab %} <--  does not yet work
#   {% endtabs %}
#   {% tabcontent %}
#         ...
#   {% endtabcontent %}
# {% endtabpage %}
from django.template import Library, Node, loader
from django.conf import settings

register = Library()


class TabPageNode(Node):
    def __init__(self, tabs_nodelist, tabcontent_nodelist, no_ajax):
        self.tabs_nodelist = tabs_nodelist
        self.tabcontent_nodelist = tabcontent_nodelist
        self.no_ajax = no_ajax

    def render(self, context):
        # Render child nodelists
        tabs = (self.tabs_nodelist.render(context) if self.tabs_nodelist else '')
        content = (self.tabcontent_nodelist.render(context) if self.tabcontent_nodelist else '')

        return loader.get_template('tabbing/default-style.html').render(
                    {
                            'tabs': tabs,
                            'content': content,
                            'MEDIA_URL': getattr(settings, 'MEDIA_URL', ''),
                            'STATIC_URL': getattr(settings, 'STATIC_URL', ''),
                            'no_ajax': self.no_ajax,
                            })

    def __iter__(self):
        for node in self.tabs_nodelist:
            yield node
        for node in self.tabcontent_nodelist:
            yield node

    def get_nodes_by_type(self, nodetype):
        nodes = []
        if isinstance(self, nodetype):
            nodes.append(self)
        nodes.extend(self.tabs_nodelist.get_nodes_by_type(nodetype))
        nodes.extend(self.tabcontent_nodelist.get_nodes_by_type(nodetype))
        return nodes


@register.tag
def tabpage(parser, token):
    # Nodelist
    tabs_nodelist = None
    tabcontent_nodelist = None

    bits = token.contents.split()
    no_ajax = 'no_ajax' in bits

    while True:
        parser.delete_first_token()
        parser.parse(('tabs', 'tabcontent', 'endtabpage', ))

        t = parser.tokens[0].contents

        if t == 'tabs':
            parser.delete_first_token()
            tabs_nodelist = parser.parse(('endtabs',))
        elif t == 'tabcontent':
            parser.delete_first_token()
            tabcontent_nodelist = parser.parse(('endtabcontent',))
        elif t == 'endtabpage':
            parser.delete_first_token()
            return TabPageNode(tabs_nodelist, tabcontent_nodelist, no_ajax)
        else:
            raise Exception('Found unexpected token {}'.format(t))



"""
The Paginator and Page classes have been copied from django/core/paginator.py
But little changes were applied to fit our needs.

The paginate method and template tags are own design.
"""
from math import ceil


class Paginator(object):
    def __init__(self, object_list, per_page, page_variable, allow_empty_first_page=True, object_count=None):
        self.object_list = object_list
        self.per_page = per_page
        self.page_variable = page_variable
        self.allow_empty_first_page = allow_empty_first_page
        self._count = object_count
        self._num_pages = None

    def page(self, number, default_to_last_page=False):
        # When we default to the last page. We want the last page to be filled, while
        # the first one may contain orphan items.
        if default_to_last_page:
            top = self.count - self.per_page * (self.num_pages - number)
            bottom = top - self.per_page
            if bottom < 0:
                bottom = 0
        else:
            bottom = (number - 1) * self.per_page
            top = bottom + self.per_page
            if top >= self.count:
                top = self.count

        return Page(self.object_list[bottom:top], number, self)

    @property
    def count(self):
        'Returns the total number of objects, across all pages.'
        if self._count is None:
            try:
                self._count = self.object_list.count()
            except (AttributeError, TypeError):
                # AttributeError if object_list has no count() method.
                # TypeError if object_list.count() requires arguments
                # (i.e. is of type list).
                self._count = len(self.object_list)
        return self._count

    @property
    def num_pages(self):
        'Returns the total number of pages.'
        if self._num_pages is None:
            if self.count == 0 and not self.allow_empty_first_page:
                self._num_pages = 0
            else:
                hits = max(1, self.count)
                self._num_pages = int(ceil(hits / float(self.per_page)))
        return self._num_pages

    @property
    def page_range(self):
        """
        Returns a 1-based range of pages for iterating through within
        a template for loop.
        """
        return range(1, self.num_pages + 1)


class Page(object):
    def __init__(self, object_list, number, paginator):
        self.object_list = object_list
        self.number = number
        self.paginator = paginator

    def __repr__(self):
        return '<Page {} of {}>'.format(self.number, self.paginator.num_pages)

    def has_next(self):
        return self.number < self.paginator.num_pages

    def has_previous(self):
        return self.number > 1

    def has_other_pages(self):
        return self.has_previous() or self.has_next()

    def next_page_number(self):
        return self.number + 1

    def previous_page_number(self):
        return self.number - 1

    def start_index(self):
        """
        Returns the 1-based index of the first object on this page,
        relative to total objects in the paginator.
        """
        # Special case, return zero if no items.
        if self.paginator.count == 0:
            return 0
        return (self.paginator.per_page * (self.number - 1)) + 1

    def end_index(self):
        """
        Returns the 1-based index of the last object on this page,
        relative to total objects found (hits).
        """
        # Special case for the last page because there can be orphans.
        if self.number == self.paginator.num_pages:
            return self.paginator.count
        return self.number * self.paginator.per_page


def paginate(request,
             object_list,
             num_per_page=10,
             object_count=None,
             query_string_parameters=None,
             page_variable='page',
             default_to_last_page=False,
             use_get_parameters=False):
    """
    Paginate an object list. Wrapper around the Django paginator.
    """

    paginator = Paginator(object_list, num_per_page, object_count=object_count, page_variable=page_variable)

    # Page number?
    page_num = request.GET.get(page_variable, None)
    number_of_pages = paginator.num_pages

    if page_num:
        try:
            page_num = int(page_num)
            page_num = max(page_num, 1)  # Make sure page number isn't negative.
            page_num = min(page_num, number_of_pages)  # Make sure the pagenumber isn't bigger than the number of pages.
        except ValueError:
            page_num = 1
    else:
        # Default start page
        if default_to_last_page:
            page_num = number_of_pages
        else:
            page_num = 1

    # Create output page instance
    output_page = paginator.page(page_num, default_to_last_page)

    # Query string parameters
    output_page.additional_parameters = query_string_parameters or {}

    output_page.page_variable = page_variable
    output_page.use_get_parameters = use_get_parameters
    output_page.request = request

    return output_page



from math import floor

from django import template
from django.conf import settings
from django.template.loader import render_to_string

register = template.Library()


@register.tag('paginate')
def paginator(parser, token):
    """
    {% paginate paginator_object nopreload %} content nodes {% endpaginate %}
    """
    args = token.split_contents()

    # Parameters
    page = args[1]

    preload = True
    for a in args[1:]:
        if a == 'nopreload':
            preload = False

    # Read nodelist
    nodelist = parser.parse(('endpaginate',))
    parser.delete_first_token()

    # Return meta node
    return PaginateNode(page, nodelist, preload)


@register.simple_tag
def paginated_querystring(page, page_number):
    """
    Returns the http query part of the url with a given page number
    :param page_number:
    """
    return '?{}={}'.format(page, page_number)


class PaginateNode(template.Node):
    def __init__(self, page, nodelist, preload=True):
        self.page = template.Variable(page)
        self.nodelist = nodelist
        self.preload = preload

    def render(self, context):
        page = self.page.resolve(context)

        if not page:
            return u''

        context = {
                'request': page.request,
                'preload': self.preload,
                'current': page.number,
                'total': page.paginator.num_pages,
                'body': self.nodelist.render(context),
                'page': self.page,
                'page_variable': page.paginator.page_variable,
                'prev_page': page.previous_page_number() if page.has_previous() else None,
                'next_page': page.next_page_number() if page.has_next() else None,
                'MEDIA_URL': getattr(settings, 'MEDIA_URL', ''),
                'STATIC_URL': getattr(settings, 'STATIC_URL', ''),
            }

        style = getattr(settings, 'PAGINATION_STYLE', None)
        if style and style == 'verbose':
            context.update(self._verbose_context(page.number, page.paginator.num_pages))
            return render_to_string('pagination/paginate-verbose.html', context)
        else:
            return render_to_string('pagination/paginate.html', context)

    def _verbose_context(self, number, num_pages):
        main_range = map(int, [floor(number-5 / 2.0) + 1, floor(number + 5 / 2.0)])

        if main_range[0] < 1:
            main_range = map(abs(main_range[0] - 1).__add__, main_range)
        if main_range[1] > num_pages:
            main_range = map((num_pages - main_range[1]).__add__, main_range)

        if main_range[0] <= 1:
            leading = []
            main_range = [1, max(5, min(number, main_range[1]))]
            main_range[0] = 1
        else:
            leading = range(1, 2)

        if main_range[1] >= num_pages:
            trailing = []
            if not leading:
                main_range = [1, num_pages]
            else:
                main_range = [min(num_pages - 4, max(number, main_range[0])), num_pages]
        else:
            trailing = range(num_pages, num_pages+1)

        main_range = [max(main_range[0], 1), min(main_range[1], num_pages)]

        main_range = range(main_range[0], main_range[1] + 1)

        return {'main_range': main_range, 'leading_range': leading, 'trailing_range': trailing,
                'page_range': reduce(lambda x, y: x+((x and y) and [False]) + y, [leading, main_range, trailing])}



from setuptools import setup, find_packages

setup(
    name='django-ajax-utilities',
    version='1.2.9',
    url='https://github.com/vikingco/django-ajax-utilities',
    license='BSD',
    description='Pagination, xhr and tabbing utilities for the Django framework.',
    long_description=open('README', 'r').read(),
    author='Unleashed NV',
    author_email='operations@unleashed.be',
    packages=find_packages('.'),
    package_data={'django_ajax': [
                'static/*.js', 'static/*/*.js', 'static/*/*/*.js',
                'static/*.css', 'static/*/*.css', 'static/*/*/*.css',
                'static/*.png', 'static/*/*.png', 'static/*/*/*.png', 'static/*/*/*/*.png',
                'static/*.gif', 'static/*/*.gif', 'static/*/*/*.gif', 'static/*/*/*/*.gif',
                'templates/*.html', 'templates/*/*.html', 'templates/*/*/*.html'
                ]},
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
)
